from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import include, path

from django.contrib import admin

admin.autodiscover()

urlpatterns = staticfiles_urlpatterns() + [
    path("admin/", admin.site.urls),
    path("", include(("posts.urls", "posts"))),
]



from django.urls import include, re_path

from .views import PostListView, PostDetailView

urlpatterns = [
    re_path(r"(?P<pk>\d+)/$", PostDetailView.as_view(), name="detail"),
    re_path(r"$", PostListView.as_view(), name="list"),
]



import re
import urllib.parse as urlparse
from unittest import TestCase
from unittest.mock import Mock, patch

from django.http import HttpRequest
from django.template import TemplateSyntaxError
from django.template.base import Template
from django.template.context import RequestContext
from django.test.client import RequestFactory

from embed_video.templatetags.embed_video_tags import VideoNode

URL_PATTERN = re.compile(r'src="?\'?([^"\'>]*)"')


class EmbedTestCase(TestCase):
    def render_template(self, template_string, context=None):
        response = RequestContext(HttpRequest(), context)
        return Template(template_string).render(response).strip()

    def assertRenderedTemplate(self, template_string, output, context=None):
        rendered_output = self.render_template(template_string, context=context)
        self.assertEqual(rendered_output, output.strip())

    def url_dict(self, url):
        """
        Parse the URL into a format suitable for comparison, ignoring the query
        parameter order.
        """

        parsed = urlparse.urlparse(url)
        query = urlparse.parse_qs(parsed.query)

        return {
            "scheme": parsed.scheme,
            "netloc": parsed.netloc,
            "path": parsed.path,
            "params": parsed.params,
            "query": query,
            "fragment": parsed.fragment,
        }

    def assertUrlEqual(self, actual, expected, msg=None):
        """Assert two URLs are equal, ignoring the query parameter order."""
        actual_dict = self.url_dict(actual)
        expected_dict = self.url_dict(expected)

        self.assertEqual(actual_dict, expected_dict, msg=msg)

    def test_embed(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {% video ytb 'large' %}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="960" height="720" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_embed_invalid_url(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/edit?abcd=efgh' as ytb %}
                {{ ytb.url }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(template, "")

    def test_embed_with_none_instance(self):
        template = """
            {% with None as my_video %}
            {% load embed_video_tags %}
            {% video my_video %}{% endwith %}
        """
        self.assertRenderedTemplate(template, "")

    def test_embed_empty_string(self):
        template = """
            {% load embed_video_tags %}
            {% video '' 'large' %}
        """
        self.assertRenderedTemplate(template, "")

    def test_direct_embed_tag(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://www.youtube.com/watch?v=jsrRJyHBvzw" "large" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="960" height="720" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_direct_embed_tag_with_default_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://www.youtube.com/watch?v=jsrRJyHBvzw" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="480" height="360" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_direct_embed_invalid_url(self):
        template = """
            {% load embed_video_tags %}
            {% video "https://soundcloud.com/xyz/foo" %}
        """
        self.assertRenderedTemplate(template, "")

    def test_user_size(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {% video ytb '800x800' %}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="800" height="800" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_wrong_size(self):
        template = Template(
            """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' 'so x huge' %}
        """
        )
        request = RequestContext(HttpRequest())
        self.assertRaises(TemplateSyntaxError, template.render, request)

    def test_tag_youtube(self):
        template = """
            {% load embed_video_tags %}
            {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' as ytb %}
                {{ ytb.url }} {{ ytb.backend }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            "https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque " "YoutubeBackend",
        )

    def test_tag_vimeo(self):
        template = """
            {% load embed_video_tags %}
            {% video 'https://vimeo.com/72304002' as vimeo %}
                {{ vimeo.url }} {{ vimeo.backend }} {{ vimeo.info.duration }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template, "https://player.vimeo.com/video/72304002 VimeoBackend 176"
        )

    def test_tag_soundcloud(self):
        template = """
            {% load embed_video_tags %}
            {% video 'https://soundcloud.com/community/soundcloud-case-study-wildlife' as soundcloud %}
                {{ soundcloud.url }} {{ soundcloud.backend }}
            {% endvideo %}
        """
        self.assertRenderedTemplate(
            template,
            "https://w.soundcloud.com/player/?visual=true&amp;url=https%3A%2F%2Fapi.soundcloud.com%2Ftracks%2F82244706&amp;show_artwork=true "
            "SoundCloudBackend",
        )

    @patch("embed_video.backends.EMBED_VIDEO_TIMEOUT", 0.000001)
    @patch("urllib3.connectionpool.log")
    @patch("embed_video.templatetags.embed_video_tags.logger")
    def test_empty_if_timeout(self, embed_video_logger, urllib_logger):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" as my_video %}
                {{ my_video.thumbnail }}
            {% endvideo %}
        """

        self.assertRenderedTemplate(template, "")

        urllib_logger.debug.assert_called_with(
            "Starting new HTTPS connection (%d): %s:%s", 1, "vimeo.com", 443
        )

        embed_video_logger.exception.assert_called_with(
            "Timeout reached during rendering embed video (`http://vimeo.com/72304002`)"
        )

    def test_relative_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" "80%x30%" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="80%" height="30%" '
            'src="https://player.vimeo.com/video/72304002" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_allow_spaces_in_size(self):
        template = """
            {% load embed_video_tags %}
            {% video "http://vimeo.com/72304002" "80% x 300" %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="80%" height="300" '
            'src="https://player.vimeo.com/video/72304002" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_embed_with_query(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' query="rel=1&wmode=transparent" as ytb %}
               {{ ytb.url }}
           {% endvideo %}
        """

        output = self.render_template(template)
        self.assertUrlEqual(
            output, "https://www.youtube.com/embed/jsrRJyHBvzw?rel=1&wmode=transparent"
        )

    def test_direct_embed_with_query(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' query="rel=1&wmode=transparent" %}
        """

        output = self.render_template(template)

        # The order of query parameters in the URL might change between Python
        # versions. Compare the URL and the outer part separately.

        url_pattern = re.compile(r'http[^"]+')
        url = url_pattern.search(output).group(0)

        self.assertUrlEqual(
            url, "https://www.youtube.com/embed/jsrRJyHBvzw?rel=1&wmode=transparent"
        )

        output_without_url = url_pattern.sub("URL", output)

        self.assertEqual(
            output_without_url,
            '<iframe width="480" height="360" '
            'src="URL" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_set_options(self):
        template = """
           {% load embed_video_tags %}
           {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' "300x200" is_secure=True query="rel=1" %}
       """
        self.assertRenderedTemplate(
            template,
            '<iframe width="300" height="200" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?rel=1" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )

    def test_size_as_variable(self):
        template = """
            {% load embed_video_tags %}
            {% with size="500x200" %}
                {% video 'http://www.youtube.com/watch?v=jsrRJyHBvzw' size %}
            {% endwith %}
        """
        self.assertRenderedTemplate(
            template,
            '<iframe width="500" height="200" '
            'src="https://www.youtube.com/embed/jsrRJyHBvzw?wmode=opaque" '
            'loading="lazy" frameborder="0" allowfullscreen></iframe>',
        )


class EmbedVideoNodeTestCase(TestCase):
    def setUp(self):
        self.parser = Mock()
        self.token = Mock(methods=["split_contents"])

    def test_repr(self):
        self.token.split_contents.return_value = (
            "video",
            "http://youtu.be/v/1234",
            "as",
            "myvideo",
        )
        self.parser.compile_filter.return_value = "some_url"

        node = VideoNode(self.parser, self.token)
        self.assertEqual(str(node), '<VideoNode "some_url">')

    def test_videonode_iter(self):
        out = ["a", "b", "c", "d"]

        class FooNode(VideoNode):
            nodelist_file = out

            def __init__(self):
                pass

        node = FooNode()
        self.assertEqual(out, [x for x in node])

    def test_get_backend_secure(self):
        class SecureRequest(RequestFactory):
            is_secure = lambda x: True

        context = {"request": SecureRequest()}
        backend = VideoNode.get_backend(
            "http://www.youtube.com/watch?v=jsrRJyHBvzw", context
        )
        self.assertTrue(backend.is_secure)

    def test_get_backend_insecure(self):
        class InsecureRequest(RequestFactory):
            is_secure = lambda x: False

        context = {"request": InsecureRequest()}
        backend = VideoNode.get_backend(
            "http://www.youtube.com/watch?v=jsrRJyHBvzw", context
        )
        self.assertFalse(backend.is_secure)



from unittest import TestCase
from unittest.mock import patch

import requests

from embed_video.backends import SoundCloudBackend, VideoDoesntExistException
from embed_video.tests.backends import BackendTestMixin


class SoundCloudBackendTestCase(BackendTestMixin, TestCase):
    urls = (
        ("https://soundcloud.com/community/soundcloud-case-study-wildlife", "82244706"),
        (
            "https://soundcloud.com/matej-roman/jaromir-nohavica-karel-plihal-mikymauz",
            "7834701",
        ),
        ("https://soundcloud.com/beny97/sets/jaromir-nohavica-prazska", "960591"),
        (
            "https://soundcloud.com/jet-silver/couleur-3-downtown-boogie-show-prove",
            "257485194",
        ),
    )

    instance = SoundCloudBackend

    def setUp(self):
        class FooBackend(SoundCloudBackend):
            url = "foobar"

            def get_info(self):
                return {
                    "width": 123,
                    "height": 321,
                    "thumbnail_url": "xyz",
                    "html": '\u003Ciframe width="100%" height="400" '
                    'scrolling="no" frameborder="no" '
                    'src="{0}"\u003E\u003C/iframe\u003E'.format(self.url),
                }

        self.foo = FooBackend("abcd")

    def test_width(self):
        self.assertEqual(self.foo.width, 123)

    def test_height(self):
        self.assertEqual(self.foo.height, 321)

    def test_get_thumbnail_url(self):
        self.assertEqual(self.foo.get_thumbnail_url(), "xyz")

    def test_get_url(self):
        self.assertEqual(self.foo.get_url(), self.foo.url)

    @patch("embed_video.backends.EMBED_VIDEO_TIMEOUT", 0.000001)
    def test_timeout_in_get_info(self):
        backend = SoundCloudBackend(
            "https://soundcloud.com/community/soundcloud-case-study-wildlife"
        )
        self.assertRaises(requests.Timeout, backend.get_info)

    def test_invalid_url(self):
        """Check if bug #21 is fixed."""
        backend = SoundCloudBackend("https://soundcloud.com/xyz/foo")
        self.assertRaises(VideoDoesntExistException, backend.get_info)



import json
import re
import urllib.parse as urlparse

import requests
from django.http import QueryDict
from django.template.loader import render_to_string
from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe

from embed_video.settings import (
    EMBED_VIDEO_BACKENDS,
    EMBED_VIDEO_TIMEOUT,
    EMBED_VIDEO_YOUTUBE_CHECK_THUMBNAIL,
    EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY,
)


class EmbedVideoException(Exception):
    """Parental class for all embed_video exceptions"""

    pass


class VideoDoesntExistException(EmbedVideoException):
    """Exception thrown if video doesn't exist"""

    pass


class UnknownBackendException(EmbedVideoException):
    """Exception thrown if video backend is not recognized."""

    pass


class UnknownIdException(VideoDoesntExistException):
    """
    Exception thrown if backend is detected, but video ID cannot be parsed.
    """

    pass


def detect_backend(url):
    """
    Detect the right backend for given URL.

    Goes over backends in ``settings.EMBED_VIDEO_BACKENDS``,
    calls :py:func:`~VideoBackend.is_valid` and returns backend instance.

    :param url: URL which is passed to `is_valid` methods of VideoBackends.
    :type url: str

    :return: Returns recognized VideoBackend
    :rtype: VideoBackend
    """

    for backend_name in EMBED_VIDEO_BACKENDS:
        backend = import_string(backend_name)
        if backend.is_valid(url):
            return backend(url)

    raise UnknownBackendException


class VideoBackend:
    """
    Base class used as parental class for backends.


    Backend variables:

    .. autosummary::

        url
        code
        thumbnail
        query
        info
        is_secure
        protocol
        template_name


    .. code-block:: python

        class MyBackend(VideoBackend):
            ...

    """

    re_code = None
    """
    Compiled regex (:py:func:`re.compile`) to search code in URL.

    Example: ``re.compile(r'myvideo\.com/\?code=(?P<code>\w+)')``
    """

    re_detect = None
    """
    Compilede regec (:py:func:`re.compile`) to detect, if input URL is valid
    for current backend.

    Example: ``re.compile(r'^http://myvideo\.com/.*')``
    """

    pattern_url = None
    """
    Pattern in which the code is inserted.

    Example: ``http://myvideo.com?code=%s``

    :type: str
    """

    pattern_thumbnail_url = None
    """
    Pattern in which the code is inserted to get thumbnail url.

    Example: ``http://static.myvideo.com/thumbs/%s``

    :type: str
    """

    allow_https = True
    """
    Sets if HTTPS version allowed for specific backend.

    :type: bool
    """

    template_name = "embed_video/embed_code.html"
    """
    Name of embed code template used by :py:meth:`get_embed_code`.

    Passed template variables: ``{{ backend }}`` (instance of VideoBackend),
    ``{{ width }}``, ``{{ height }}``

    :type: str
    """

    default_query = ""
    """
    Default query string or `QueryDict` appended to url

    :type: str
    """

    is_secure = False
    """
    Decides if secured protocol (HTTPS) is used.

    :type: bool
    """

    def __init__(self, url):
        """
        First it tries to load data from cache and if it don't succeed, run
        :py:meth:`init` and then save it to cache.

        :type url: str
        """
        self.backend = self.__class__.__name__
        self._url = url
        self.query = QueryDict(self.default_query, mutable=True)

    @property
    def code(self):
        """
        Code of video.
        """
        return self.get_code()

    @property
    def url(self):
        """
        URL of video.
        """
        return self.get_url()

    @property
    def protocol(self):
        """
        Protocol used to generate URL.
        """
        return "https" if self.allow_https and self.is_secure else "http"

    @property
    def thumbnail(self):
        """
        URL of video thumbnail.
        """
        return self.get_thumbnail_url()

    @property
    def info(self):
        """
        Additional information about video. Not implemented in all backends.
        """
        return self.get_info()

    @property
    def query(self):
        """
        String transformed to QueryDict appended to url.
        """
        return self._query

    @query.setter
    def query(self, value):
        """
        :type value: QueryDict | str
        """
        self._query = (
            value if isinstance(value, QueryDict) else QueryDict(value, mutable=True)
        )

    @classmethod
    def is_valid(cls, url):
        """
        Class method to control if passed url is valid for current backend. By
        default it is done by :py:data:`re_detect` regex.

        :type url: str
        """
        return True if cls.re_detect.match(url) else False

    def get_code(self):
        """
        Returns video code matched from given url by :py:data:`re_code`.

        :rtype: str
        """
        match = self.re_code.search(self._url)
        if match:
            return match.group("code")

    def get_url(self):
        """
        Returns URL folded from :py:data:`pattern_url` and parsed code.
        """
        url = self.pattern_url.format(code=self.code, protocol=self.protocol)
        url += "?" + self.query.urlencode() if self.query else ""
        return mark_safe(url)

    def get_thumbnail_url(self):
        """
        Returns thumbnail URL folded from :py:data:`pattern_thumbnail_url` and
        parsed code.

        :rtype: str
        """
        return self.pattern_thumbnail_url.format(code=self.code, protocol=self.protocol)

    def get_embed_code(self, width, height):
        """
        Returns embed code rendered from template :py:data:`template_name`.

        :type width: int | str
        :type height: int | str
        :rtype: str
        """
        return render_to_string(
            self.template_name, {"backend": self, "width": width, "height": height}
        )

    def get_info(self):
        """
        :rtype: dict
        """
        raise NotImplementedError

    def set_options(self, options):
        """
        :type options: dict
        """
        for key in options:
            setattr(self, key, options[key])


class YoutubeBackend(VideoBackend):
    """
    Backend for YouTube URLs.
    """

    re_detect = re.compile(r"^(http(s)?://)?(www\.|m\.)?youtu(\.?)be(\.com)?/.*", re.I)

    re_code = re.compile(
        r"""youtu(\.?)be(\.com)?/  # match youtube's domains
            (\#/)? # for mobile urls
            (embed/)?  # match the embed url syntax
            (v/)?
            (shorts/)? # match youtube shorts
            (watch\?v=)?  # match the youtube page url
            (ytscreeningroom\?v=)?
            (feeds/api/videos/)?
            (user\S*[^\w\-\s])?
            (?P<code>[\w\-]{11})[a-z0-9;:@?&%=+/\$_.-]*  # match and extract
        """,
        re.I | re.X,
    )

    pattern_url = "{protocol}://www.youtube.com/embed/{code}"
    pattern_thumbnail_url = "{protocol}://img.youtube.com/vi/{code}/{resolution}"
    default_query = EMBED_VIDEO_YOUTUBE_DEFAULT_QUERY
    resolutions = [
        "maxresdefault.jpg",
        "sddefault.jpg",
        "hqdefault.jpg",
        "mqdefault.jpg",
    ]

    is_secure = True
    """
    Decides if secured protocol (HTTPS) is used.

    :type: bool
    """

    def get_code(self):
        code = super().get_code()

        if not code:
            parsed_url = urlparse.urlparse(self._url)
            parsed_qs = urlparse.parse_qs(parsed_url.query)

            if "v" in parsed_qs:
                code = parsed_qs["v"][0]
            elif "video_id" in parsed_qs:
                code = parsed_qs["video_id"][0]
            else:
                raise UnknownIdException("Cannot get ID from `{0}`".format(self._url))

        return code

    def get_thumbnail_url(self):
        """
        Returns thumbnail URL folded from :py:data:`pattern_thumbnail_url` and
        parsed code.

        :rtype: str
        """
        if not EMBED_VIDEO_YOUTUBE_CHECK_THUMBNAIL:
            return self.pattern_thumbnail_url.format(
                code=self.code, protocol=self.protocol, resolution="hqdefault.jpg"
            )
        for resolution in self.resolutions:
            temp_thumbnail_url = self.pattern_thumbnail_url.format(
                code=self.code, protocol=self.protocol, resolution=resolution
            )
            if int(requests.head(temp_thumbnail_url).status_code) < 400:
                return temp_thumbnail_url
        return None


class VimeoBackend(VideoBackend):
    """
    Backend for Vimeo URLs.
    """

    re_detect = re.compile(r"^((http(s)?:)?//)?(www\.)?(player\.)?vimeo\.com/.*", re.I)
    re_code = re.compile(
        r"""vimeo\.com/(video/)?(channels/(.*/)?)?((.+)/review/)?(manage/)?(?P<code>[0-9]+)""",
        re.I,
    )
    pattern_url = "{protocol}://player.vimeo.com/video/{code}"
    pattern_info = "{protocol}://vimeo.com/api/v2/video/{code}.json"

    is_secure = True
    """
    Decides if secured protocol (HTTPS) is used.

    :type: bool
    """

    def get_info(self):
        try:
            response = requests.get(
                self.pattern_info.format(code=self.code, protocol=self.protocol),
                timeout=EMBED_VIDEO_TIMEOUT,
            )
            return json.loads(response.text)[0]
        except ValueError:
            raise VideoDoesntExistException()

    def get_thumbnail_url(self):
        return self.info.get("thumbnail_large")


class SoundCloudBackend(VideoBackend):
    """
    Backend for SoundCloud URLs.
    """

    base_url = "{protocol}://soundcloud.com/oembed"

    re_detect = re.compile(r"^(http(s)?://(www\.|m\.)?)?soundcloud\.com/.*", re.I)
    re_code = re.compile(r'src=".*%2F(?P<code>\d+)&show_artwork.*"', re.I)
    re_url = re.compile(r'src="(?P<url>.*?)"', re.I)

    is_secure = True
    """
    Decides if secured protocol (HTTPS) is used.

    :type: bool
    """

    @cached_property
    def width(self):
        """
        :rtype: str
        """
        return self.info.get("width")

    @cached_property
    def height(self):
        """
        :rtype: str
        """
        return self.info.get("height")

    def get_info(self):
        params = {"format": "json", "url": self._url}
        r = requests.get(
            self.base_url.format(protocol=self.protocol),
            params=params,
            timeout=EMBED_VIDEO_TIMEOUT,
        )

        if r.status_code != 200:
            raise VideoDoesntExistException(
                "SoundCloud returned status code `{0}`.".format(r.status_code)
            )

        return json.loads(r.text)

    def get_thumbnail_url(self):
        return self.info.get("thumbnail_url")

    def get_url(self):
        match = self.re_url.search(self.info.get("html"))
        return match.group("url")

    def get_code(self):
        match = self.re_code.search(self.info.get("html"))
        return match.group("code")

    def get_embed_code(self, width, height):
        return super().get_embed_code(width=width, height=height)



from django.views.generic import ListView, DetailView

from .models import Post


class PostListView(ListView):
    model = Post
    paginate_by = 10


class PostDetailView(DetailView):
    model = Post



from embed_video.backends import detect_backend


class BackendTestMixin:
    urls = []
    instance = None

    def test_detect(self):
        for url in self.urls:
            backend = detect_backend(url[0])
            self.assertIsInstance(backend, self.instance)

    def test_code(self):
        for url in self.urls:
            backend = self.instance(url[0])
            self.assertEqual(backend.code, url[1])



from unittest import TestCase

from django.test import SimpleTestCase

from embed_video.admin import AdminVideoMixin, AdminVideoWidget
from embed_video.backends import VimeoBackend
from embed_video.fields import EmbedVideoField, EmbedVideoFormField


class AdminVideoWidgetTestCase(SimpleTestCase):
    def test_size(self):
        widget = AdminVideoWidget()
        self.assertTrue("size" in widget.attrs)

    def test_overwrite_size(self):
        widget = AdminVideoWidget(attrs={"size": 0})
        self.assertEqual(widget.attrs["size"], 0)

    def test_add_to_attrs(self):
        widget = AdminVideoWidget(attrs={"name": "foo"})
        self.assertEqual(widget.attrs["name"], "foo")
        self.assertTrue("size" in widget.attrs)

    def test_render_empty_value(self):
        widget = AdminVideoWidget(attrs={"size": "0"})
        self.assertHTMLEqual(
            widget.render("foo"), '<input name="foo" size="0" type="text" />'
        )

    def test_render(self):
        backend = VimeoBackend("https://vimeo.com/1")
        widget = AdminVideoWidget(attrs={"size": "0"})
        widget.output_format = "{video}{input}"

        self.assertHTMLEqual(
            widget.render("foo", backend.url, size=(100, 100)),
            backend.get_embed_code(100, 100)
            + '<input name="foo" size="0" type="text" value="%s" />' % backend.url,
        )

    def test_render_unknown_backend(self):
        widget = AdminVideoWidget()
        self.assertHTMLEqual(
            widget.render("foo", "abcd"),
            '<input name="foo" size="40" type="text" value="abcd" />',
        )

    def test_render_video_doesnt_exist(self):
        widget = AdminVideoWidget()
        self.assertHTMLEqual(
            widget.render("foo", "https://soundcloud.com/xyz/foo"),
            '<input name="foo" size="40" type="text" value="https://soundcloud.com/xyz/foo" />',
        )


class AdminVideoMixinTestCase(TestCase):
    def test_embedvideofield(self):
        foo = EmbedVideoField()
        mixin = AdminVideoMixin()
        self.assertTrue(
            isinstance(mixin.formfield_for_dbfield(foo), EmbedVideoFormField)
        )

    def test_other_fields(self):
        class Parent:
            def formfield_for_dbfield(*args, **kwargs):
                raise Exception

        class MyAdmin(AdminVideoMixin, Parent):
            pass

        myadmin = MyAdmin()
        self.assertRaises(Exception, myadmin.formfield_for_dbfield, "foo")



"""
Sphinx plugins for Django documentation.
"""
import json
import os
import re

from sphinx import addnodes, __version__ as sphinx_ver
from sphinx.builders.html import StandaloneHTMLBuilder
from sphinx.writers.html import HTMLTranslator
from sphinx.util.console import bold
from docutils.parsers.rst import Directive

# RE for option descriptions without a '--' prefix
simple_option_desc_re = re.compile(r"([-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)")


def setup(app):
    app.add_crossref_type(
        directivename="setting", rolename="setting", indextemplate="pair: %s; setting"
    )
    app.add_crossref_type(
        directivename="templatetag",
        rolename="ttag",
        indextemplate="pair: %s; template tag",
    )
    app.add_crossref_type(
        directivename="templatefilter",
        rolename="tfilter",
        indextemplate="pair: %s; template filter",
    )
    app.add_crossref_type(
        directivename="fieldlookup",
        rolename="lookup",
        indextemplate="pair: %s; field lookup type",
    )
    app.add_description_unit(
        directivename="django-admin",
        rolename="djadmin",
        indextemplate="pair: %s; django-admin command",
        parse_node=parse_django_admin_node,
    )
    app.add_description_unit(
        directivename="django-admin-option",
        rolename="djadminopt",
        indextemplate="pair: %s; django-admin command-line option",
        parse_node=parse_django_adminopt_node,
    )
    app.add_config_value("django_next_version", "0.0", True)
    app.add_directive("versionadded", VersionDirective)
    app.add_directive("versionchanged", VersionDirective)
    app.add_builder(DjangoStandaloneHTMLBuilder)


class VersionDirective(Directive):
    has_content = True
    required_arguments = 1
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        if len(self.arguments) > 1:
            msg = """Only one argument accepted for directive '{directive_name}::'.
            Comments should be provided as content,
            not as an extra argument.""".format(
                directive_name=self.name
            )
            raise self.error(msg)

        env = self.state.document.settings.env
        ret = []
        node = addnodes.versionmodified()
        ret.append(node)

        if self.arguments[0] == env.config.django_next_version:
            node["version"] = "Development version"
        else:
            node["version"] = self.arguments[0]

        node["type"] = self.name
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        env.note_versionchange(node["type"], node["version"], node, self.lineno)
        return ret


class DjangoHTMLTranslator(HTMLTranslator):
    """
    Django-specific reST to HTML tweaks.
    """

    # Don't use border=1, which docutils does by default.
    def visit_table(self, node):
        self.context.append(self.compact_p)
        self.compact_p = True
        self._table_row_index = 0  # Needed by Sphinx
        self.body.append(self.starttag(node, "table", CLASS="docutils"))

    def depart_table(self, node):
        self.compact_p = self.context.pop()
        self.body.append("</table>\n")

    # <big>? Really?
    def visit_desc_parameterlist(self, node):
        self.body.append("(")
        self.first_param = 1
        self.param_separator = node.child_text_separator

    def depart_desc_parameterlist(self, node):
        self.body.append(")")

    if sphinx_ver < "1.0.8":
        #
        # Don't apply smartypants to literal blocks
        #
        def visit_literal_block(self, node):
            self.no_smarty += 1
            HTMLTranslator.visit_literal_block(self, node)

        def depart_literal_block(self, node):
            HTMLTranslator.depart_literal_block(self, node)
            self.no_smarty -= 1

    #
    # Turn the "new in version" stuff (versionadded/versionchanged) into a
    # better callout -- the Sphinx default is just a little span,
    # which is a bit less obvious that I'd like.
    #
    # FIXME: these messages are all hardcoded in English. We need to change
    # that to accommodate other language docs, but I can't work out how to make
    # that work.
    #
    version_text = {
        "deprecated": "Deprecated in Django %s",
        "versionchanged": "Changed in Django %s",
        "versionadded": "New in Django %s",
    }

    def visit_versionmodified(self, node):
        self.body.append(self.starttag(node, "div", CLASS=node["type"]))
        title = "%s%s" % (
            self.version_text[node["type"]] % node["version"],
            ":" if len(node) else ".",
        )
        self.body.append('<span class="title">%s</span> ' % title)

    def depart_versionmodified(self, node):
        self.body.append("</div>\n")

    # Give each section a unique ID -- nice for custom CSS hooks
    def visit_section(self, node):
        old_ids = node.get("ids", [])
        node["ids"] = ["s-" + i for i in old_ids]
        node["ids"].extend(old_ids)
        HTMLTranslator.visit_section(self, node)
        node["ids"] = old_ids


def parse_django_admin_node(env, sig, signode):
    command = sig.split(" ")[0]
    env._django_curr_admin_command = command
    title = "django-admin.py %s" % sig
    signode += addnodes.desc_name(title, title)
    return sig


def parse_django_adminopt_node(env, sig, signode):
    """A copy of sphinx.directives.CmdoptionDesc.parse_signature()"""
    from sphinx.domains.std import option_desc_re

    count = 0
    firstname = ""
    for m in option_desc_re.finditer(sig):
        optname, args = m.groups()
        if count:
            signode += addnodes.desc_addname(", ", ", ")
        signode += addnodes.desc_name(optname, optname)
        signode += addnodes.desc_addname(args, args)
        if not count:
            firstname = optname
        count += 1
    if not count:
        for m in simple_option_desc_re.finditer(sig):
            optname, args = m.groups()
            if count:
                signode += addnodes.desc_addname(", ", ", ")
            signode += addnodes.desc_name(optname, optname)
            signode += addnodes.desc_addname(args, args)
            if not count:
                firstname = optname
            count += 1
    if not firstname:
        raise ValueError
    return firstname


class DjangoStandaloneHTMLBuilder(StandaloneHTMLBuilder):
    """
    Subclass to add some extra things we need.
    """

    name = "djangohtml"

    def finish(self):
        super(DjangoStandaloneHTMLBuilder, self).finish()
        self.info(bold("writing templatebuiltins.js..."))
        xrefs = self.env.domaindata["std"]["objects"]
        templatebuiltins = {
            "ttags": [
                n
                for ((t, n), (l, a)) in xrefs.items()
                if t == "templatetag" and l == "ref/templates/builtins"
            ],
            "tfilters": [
                n
                for ((t, n), (l, a)) in xrefs.items()
                if t == "templatefilter" and l == "ref/templates/builtins"
            ],
        }
        outfilename = os.path.join(self.outdir, "templatebuiltins.js")
        with open(outfilename, "w") as fp:
            fp.write("var django_template_builtins = ")
            json.dump(templatebuiltins, fp)
            fp.write(";\n")



# Generated by Django 4.0.5 on 2022-06-26 08:55

from django.db import migrations, models
import embed_video.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=50)),
                ('video', embed_video.fields.EmbedVideoField(help_text='This is a help text', verbose_name='My video')),
            ],
        ),
    ]



# Django settings for example_project project.
import django

DEBUG = True

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "example_project.sqlite3",
    }
}

SITE_ID = 1

SECRET_KEY = "u%38dln@$1!7w#cxi4np504^sa3_skv5aekad)jy_u0v2mc+nr"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

APPEND_SLASH = True

ROOT_URLCONF = "example_project.urls"

STATIC_URL = "/static/"

DJANGO_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.admin",
)

THIRD_PARTY_APPS = ("embed_video",)

LOCAL_APPS = ("example_project", "posts")

INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS



#!/usr/bin/env python

from setuptools import find_packages, setup

setup(
    name="django-embed-video",
    packages=find_packages(),
    package_data={"embed_video": ["templates/embed_video/*.html"]},
    use_scm_version=True,
    author="Cedric Carrard",
    author_email="cedric.carrard@gmail.com",
    url="https://github.com/jazzband/django-embed-video",
    description="Django app for easy embedding YouTube and Vimeo videos and music from SoundCloud.",
    long_description="\n".join(
        [
            open("README.rst", encoding="utf-8").read(),
            open("CHANGES.rst", encoding="utf-8").read(),
        ]
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Environment :: Plugins",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: WWW/HTTP",
    ],
    keywords=["youtube", "vimeo", "video", "soundcloud"],
    install_requires=["requests >= 2.19", "Django >= 2.2"],
    setup_requires=["readme", "setuptools_scm"],
    tests_require=["Django", "requests >= 2.19", "coverage"],
)



import os

DEBUG = True
SECRET_KEY = "testing_key123"

STATIC_ROOT = MEDIA_ROOT = os.path.join(os.path.dirname(__file__), "static")
STATIC_URL = MEDIA_URL = "/static/"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

INSTALLED_APPS = ("django.contrib.contenttypes", "django.contrib.auth", "embed_video")


EMBED_VIDEO_BACKENDS = (
    "embed_video.backends.YoutubeBackend",
    "embed_video.backends.VimeoBackend",
    "embed_video.backends.SoundCloudBackend",
    "embed_video.tests.backends.tests_custom_backend.CustomBackend",
)


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {"console": {"level": "DEBUG", "class": "logging.StreamHandler"}},
    "loggers": {"less": {"handlers": ["console"], "level": "DEBUG"}},
}
